@@ -1,26 +1,47 @@
 /*******************************************************************************
 
-License: 
-This software was developed at the National Institute of Standards and 
-Technology (NIST) by employees of the Federal Government in the course 
-of their official duties. Pursuant to title 17 Section 105 of the 
-United States Code, this software is not subject to copyright protection 
-and is in the public domain. NIST assumes no responsibility  whatsoever for 
-its use by other parties, and makes no guarantees, expressed or implied, 
-about its quality, reliability, or any other characteristic. 
+License:
+This software and/or related materials was developed at the National Institute
+of Standards and Technology (NIST) by employees of the Federal Government
+in the course of their official duties. Pursuant to title 17 Section 105
+of the United States Code, this software is not subject to copyright
+protection and is in the public domain.
 
-Disclaimer: 
-This software was developed to promote biometric standards and biometric
-technology testing for the Federal Government in accordance with the USA
-PATRIOT Act and the Enhanced Border Security and Visa Entry Reform Act.
-Specific hardware and software products identified in this software were used
-in order to perform the software development.  In no case does such
-identification imply recommendation or endorsement by the National Institute
-of Standards and Technology, nor does it imply that the products and equipment
-identified are necessarily the best available for the purpose.  
+This software and/or related materials have been determined to be not subject
+to the EAR (see Part 734.3 of the EAR for exact details) because it is
+a publicly available technology and software, and is freely distributed
+to any interested party with no licensing requirements.  Therefore, it is
+permissible to distribute this software as a free download from the internet.
+
+Disclaimer:
+This software and/or related materials was developed to promote biometric
+standards and biometric technology testing for the Federal Government
+in accordance with the USA PATRIOT Act and the Enhanced Border Security
+and Visa Entry Reform Act. Specific hardware and software products identified
+in this software were used in order to perform the software development.
+In no case does such identification imply recommendation or endorsement
+by the National Institute of Standards and Technology, nor does it imply that
+the products and equipment identified are necessarily the best available
+for the purpose.
+
+This software and/or related materials are provided "AS-IS" without warranty
+of any kind including NO WARRANTY OF PERFORMANCE, MERCHANTABILITY,
+NO WARRANTY OF NON-INFRINGEMENT OF ANY 3RD PARTY INTELLECTUAL PROPERTY
+or FITNESS FOR A PARTICULAR PURPOSE or for any purpose whatsoever, for the
+licensed product, however used. In no event shall NIST be liable for any
+damages and/or costs, including but not limited to incidental or consequential
+damages of any kind, including economic damage or injury to property and lost
+profits, regardless of whether NIST shall be advised, have reason to know,
+or in fact shall know of the possibility.
+
+By using this software, you agree to bear all risk relating to quality,
+use and performance of the software and/or related materials.  You agree
+to hold the Government harmless from any claim arising from your use
+of the software.
 
 *******************************************************************************/
 
+
 /***********************************************************************
       LIBRARY: LFS - NIST Latent Fingerprint System
 
@@ -37,6 +58,7 @@ identified are necessarily the best available for the purpose.
                ROUTINES:
                         init_dir2rad()
                         init_dftwaves()
+                        get_max_padding()
                         get_max_padding_V2()
                         init_rotgrids()
                         alloc_dir_powers()
@@ -44,7 +66,6 @@ identified are necessarily the best available for the purpose.
 ***********************************************************************/
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <lfs.h>
 
 /*************************************************************************
@@ -242,6 +263,33 @@ int init_dftwaves(DFTWAVES **optr, const double *dft_coefs,
    return(0);
 }
 
+/*************************************************************************
+**************************************************************************
+#cat: get_max_padding - Deterines the maximum amount of image pixel padding
+#cat:                   required by all LFS processes.  Padding is currently
+#cat:                   required by the rotated grids used in DFT analyses,
+#cat:                   rotated grids used in directional binarization,
+#cat:                   and in the grid used for isotropic binarization.
+#cat:                   The NIST generalized code enables the parameters
+#cat:                   governing these processes to be redefined, so a check
+#cat:                   at runtime is required to determine which process
+#cat:                   requires the most padding.  By using the maximum as
+#cat:                   the padding factor, all processes will run safely
+#cat:                   with a single padding of the input image avoiding the
+#cat:                   need to repad for further processes.
+
+   Input:
+      imap_blocksize  - the size (in pixels) of each IMAP block in the image
+      dirbin_grid_w   - the width (in pixels) of the rotated grids used in
+                        directional binarization
+      dirbin_grid_h   - the height (in pixels) of the rotated grids used in
+                        directional binarization
+      isobin_grid_dim - the dimension (in pixels) of the square grid used in
+                        isotropic binarization
+   Return Code:
+      Non-negative - the maximum padding required for all processes
+**************************************************************************/
+
 /*************************************************************************
 **************************************************************************
 #cat: get_max_padding_V2 - Deterines the maximum amount of image pixel padding
@@ -645,6 +693,9 @@ int alloc_power_stats(int **owis, double **opowmaxs, int **opowmax_dirs,
    int *wis, *powmax_dirs;
    double *powmaxs, *pownorms;
 
+   ASSERT_SIZE_MUL(nstats, sizeof(int));
+   ASSERT_SIZE_MUL(nstats, sizeof(double));
+
    /* Allocate DFT wave index vector */
    wis = (int *)malloc(nstats * sizeof(int));
    if(wis == (int *)NULL){
@@ -677,7 +728,7 @@ int alloc_power_stats(int **owis, double **opowmaxs, int **opowmax_dirs,
       /* Free memory allocated to this point. */
       free(wis);
       free(powmaxs);
-      free(pownorms);
+      free(powmax_dirs);
       fprintf(stderr, "ERROR : alloc_power_stats : malloc : pownorms\n");
       return(-53);
    }
