@@ -1,26 +1,47 @@
 /*******************************************************************************
 
-License: 
-This software was developed at the National Institute of Standards and 
-Technology (NIST) by employees of the Federal Government in the course 
-of their official duties. Pursuant to title 17 Section 105 of the 
-United States Code, this software is not subject to copyright protection 
-and is in the public domain. NIST assumes no responsibility  whatsoever for 
-its use by other parties, and makes no guarantees, expressed or implied, 
-about its quality, reliability, or any other characteristic. 
+License:
+This software and/or related materials was developed at the National Institute
+of Standards and Technology (NIST) by employees of the Federal Government
+in the course of their official duties. Pursuant to title 17 Section 105
+of the United States Code, this software is not subject to copyright
+protection and is in the public domain.
 
-Disclaimer: 
-This software was developed to promote biometric standards and biometric
-technology testing for the Federal Government in accordance with the USA
-PATRIOT Act and the Enhanced Border Security and Visa Entry Reform Act.
-Specific hardware and software products identified in this software were used
-in order to perform the software development.  In no case does such
-identification imply recommendation or endorsement by the National Institute
-of Standards and Technology, nor does it imply that the products and equipment
-identified are necessarily the best available for the purpose.  
+This software and/or related materials have been determined to be not subject
+to the EAR (see Part 734.3 of the EAR for exact details) because it is
+a publicly available technology and software, and is freely distributed
+to any interested party with no licensing requirements.  Therefore, it is
+permissible to distribute this software as a free download from the internet.
+
+Disclaimer:
+This software and/or related materials was developed to promote biometric
+standards and biometric technology testing for the Federal Government
+in accordance with the USA PATRIOT Act and the Enhanced Border Security
+and Visa Entry Reform Act. Specific hardware and software products identified
+in this software were used in order to perform the software development.
+In no case does such identification imply recommendation or endorsement
+by the National Institute of Standards and Technology, nor does it imply that
+the products and equipment identified are necessarily the best available
+for the purpose.
+
+This software and/or related materials are provided "AS-IS" without warranty
+of any kind including NO WARRANTY OF PERFORMANCE, MERCHANTABILITY,
+NO WARRANTY OF NON-INFRINGEMENT OF ANY 3RD PARTY INTELLECTUAL PROPERTY
+or FITNESS FOR A PARTICULAR PURPOSE or for any purpose whatsoever, for the
+licensed product, however used. In no event shall NIST be liable for any
+damages and/or costs, including but not limited to incidental or consequential
+damages of any kind, including economic damage or injury to property and lost
+profits, regardless of whether NIST shall be advised, have reason to know,
+or in fact shall know of the possibility.
+
+By using this software, you agree to bear all risk relating to quality,
+use and performance of the software and/or related materials.  You agree
+to hold the Government harmless from any claim arising from your use
+of the software.
 
 *******************************************************************************/
 
+
 /***********************************************************************
       LIBRARY: LFS - NIST Latent Fingerprint System
 
@@ -47,7 +68,6 @@ identified are necessarily the best available for the purpose.
 ***********************************************************************/
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <lfs.h>
 
 /*************************************************************************
@@ -142,7 +162,7 @@ int minmaxs(int **ominmax_val, int **ominmax_type, int **ominmax_i,
    int i, diff, state, start, loc;
    int *minmax_val, *minmax_type, *minmax_i, minmax_alloc, minmax_num;
 
-   
+
    /* Determine maximum length for allocation of buffers. */
    /* If there are fewer than 3 items ...                */
    if(num < 3){
@@ -427,27 +447,6 @@ int  in_int_list(const int item, const int *list, const int len)
       Zero      - successful completion
       Negative  - system error
 **************************************************************************/
-int remove_from_int_list(const int index, int *list, const int num)
-{
-   int fr, to;
-
-   /* Make sure the requested index is within range. */
-   if((index < 0) && (index >= num)){
-      fprintf(stderr, "ERROR : remove_from_int_list : index out of range\n");
-      return(-370);
-   }
-
-   /* Slide the remaining list of integers up over top of the */
-   /* position of the integer being removed.                  */
-   for(to = index, fr = index+1; fr < num; to++, fr++)
-      list[to] = list[fr];
-
-   /* NOTE: Decrementing the number of integers remaining in the list is */
-   /*       the responsibility of the caller!                            */
-
-   /* Return normally. */
-   return(0);
-}
 
 /*************************************************************************
 **************************************************************************
@@ -561,7 +560,7 @@ int line2direction(const int fx, const int fy,
    /* Make sure on range [0..(ndirsX2)]. */
    idir %= full_ndirs;
 
-   /* Return the integer direction. */   
+   /* Return the integer direction. */
    return(idir);
 }
 
